/*
********************************************************************************
* COPYRIGHT(c)    ϻ, 2019, 2020
* 
*        (as is).
*     .
********************************************************************************
*/



#include "audio_configuration.h"
#include "usbd_audio.h"
#include "SAI.h"



static uint8_t AudioConfiguration = 0;
static uint8_t SyncMode = 0;
static uint8_t BclkFsRatioMode = BCLK_Fs_RES_DEPENDENT;
static uint8_t SAICountUsed = 2;
static uint8_t ChannelsPairs = 2;
static uint8_t ChnlsData[CHANNELS_PREPARE_DATA_BUF_SIZE];



void SetAudioConfigDependedFuncs(AUDIO_SpeakerNode_t *speaker)
{
  uint8_t AudioConf = AudioConfiguration;
  
  if (AudioConf >= AUDIO_CONFIG_TDM_START_INDEX)
    AudioConf -= AUDIO_CONFIG_TDM_START_INDEX;
  
  switch(AudioConf)
  {
    case AUDIO_CONFIG_2_0_STEREO_32_BIT:
    case AUDIO_CONFIG_2_0_STEREO:
    default:
      speaker->SpeakerPlay = Play_SAIMaster;
      speaker->SpeakerPrepareData = 0;
    break;
    
    case AUDIO_CONFIG_3_1:
    case AUDIO_CONFIG_4_0_QUADRO:
    case AUDIO_CONFIG_7_1:
      speaker->SpeakerPlay = Play_SAIMasterAndSlave;
      speaker->SpeakerPrepareData = PrepareMultiChannelData;
    break;
  }
  
  if (AudioConfiguration >= AUDIO_CONFIG_TDM_START_INDEX)
  {
    speaker->SpeakerPlay = Play_SAIMaster;
    speaker->SpeakerPrepareData = 0;
  }
}
//------------------------------------------------------------------------------
void Play_SAIMaster(uint16_t *Data, uint16_t Size, uint8_t ResByte)
{
  if (ResByte == 3)
    ResByte = 4;
  
  uint16_t TxSize = Size / CONFIG_2_0_SAI_COUNT / ResByte;
  
  SAI_MasterDMAPrepareTx(Data, TxSize);
  SAI_MasterDMAEnable();
}
//------------------------------------------------------------------------------
void Play_SAIMasterAndSlave(uint16_t *Data, uint16_t Size, uint8_t ResByte)
{
  if (ResByte == 3)
    ResByte = 4;
  
  uint16_t TxSize = Size / ChannelsPairs / ResByte;

  SAI_MasterDMAPrepareTx(&Data[0], TxSize);
  SAI_SlaveDMAPrepareTx(&Data[(ResByte / 2) * TxSize], TxSize);
  
  SAI_SlaveDMAEnable();
  SAI_MasterDMAEnable();
}
//------------------------------------------------------------------------------
void PrepareMultiChannelData(uint8_t* AudioData, uint16_t Size, uint8_t ResInBytes)
{  
  uint16_t ChnlPairSize = Size / ChannelsPairs;
    
  if (ResInBytes == 3)
    ResInBytes = 4;
  
  uint8_t DataIncrement = ResInBytes * ChannelsPairs * 2;
  uint8_t BytesPerSAI = DataIncrement / ChannelsPairs;
  
  for (uint16_t i = 0; i < Size; i += DataIncrement)
  {
    uint16_t BaseAddr = i / ChannelsPairs;
    
    for (uint8_t ChnlPair = 0; ChnlPair < ChannelsPairs; ChnlPair++)
    {
      uint16_t ChnlPairOffset = ChnlPair * ChnlPairSize;
      uint16_t DataOffset = ChnlPair * BytesPerSAI;
      
      ChnlsData[ChnlPairOffset + BaseAddr] =     AudioData[i + DataOffset];
      ChnlsData[ChnlPairOffset + BaseAddr + 1] = AudioData[i + DataOffset + 1];
      ChnlsData[ChnlPairOffset + BaseAddr + 2] = AudioData[i + DataOffset + 2];
      ChnlsData[ChnlPairOffset + BaseAddr + 3] = AudioData[i + DataOffset + 3];
      
      if (ResInBytes == 4)
      {
        ChnlsData[ChnlPairOffset + BaseAddr + 4] = AudioData[i + DataOffset + 4];
        ChnlsData[ChnlPairOffset + BaseAddr + 5] = AudioData[i + DataOffset + 5];
        ChnlsData[ChnlPairOffset + BaseAddr + 6] = AudioData[i + DataOffset + 6];
        ChnlsData[ChnlPairOffset + BaseAddr + 7] = AudioData[i + DataOffset + 7];
      }
    }
  }
  
  memcpy(AudioData, ChnlsData, Size);
}
//------------------------------------------------------------------------------
void AudioChangeFrequency(uint32_t AudioFrequency)
{
  SAI_MasterDisable();
  if (SAICountUsed == 2)
    SAI_SlaveDisable();
  
  if (SyncMode == MASTER_INT_SYNC)
  {
    SAI_PLLSAIConfig(AudioFrequency);
    SAI_ChangeFrequency(AudioFrequency);
  }
  
  ExtSyncSelectSource(AudioFrequency);
    
  if (SAICountUsed == 2)
    SAI_SlaveEnable();
  
  SAI_MasterEnable();
}
//------------------------------------------------------------------------------
void ExtSyncSelectSource(uint32_t AudioFrequency)
{
  EXT_SYNC_SELECT_GPIO->ODR &= ~EXT_SYNC_SELECT_MASK;
    
  switch(AudioFrequency)
  {
    case USB_AUDIO_CONFIG_FREQ_44_1_K:
      EXT_SYNC_SELECT_GPIO->ODR |= EXT_SYNC_SELECT_44100;
    break;
      
    case USB_AUDIO_CONFIG_FREQ_48_K:
    default:
      EXT_SYNC_SELECT_GPIO->ODR |= EXT_SYNC_SELECT_48000;
    break;
    
    case USB_AUDIO_CONFIG_FREQ_88_2_K:
      EXT_SYNC_SELECT_GPIO->ODR |= EXT_SYNC_SELECT_88200;
    break;
    
    case USB_AUDIO_CONFIG_FREQ_96_K:
      EXT_SYNC_SELECT_GPIO->ODR |= EXT_SYNC_SELECT_96000;
    break;
    
    case USB_AUDIO_CONFIG_FREQ_176_4_K:
      EXT_SYNC_SELECT_GPIO->ODR |= EXT_SYNC_SELECT_176400;
    break;
      
    case USB_AUDIO_CONFIG_FREQ_192_K:
      EXT_SYNC_SELECT_GPIO->ODR |= EXT_SYNC_SELECT_192000;
    break;
  }
}
//------------------------------------------------------------------------------
void AudioChangeResolution(uint8_t AudioResolution)
{
  SAI_MasterDisable();
  
  if (AudioConfiguration < AUDIO_CONFIG_TDM_START_INDEX)
  {
    if (SAICountUsed == 2)
      SAI_SlaveDisable();
  
    SAI_Init_I2S(SAI_MASTER, AudioResolution, BclkFsRatioMode);
    SAI_MasterDMAChangeDataSize(AudioResolution);
  
    if (SAICountUsed == 2)
    {
      SAI_Init_I2S(SAI_SLAVE, AudioResolution, BclkFsRatioMode);
      SAI_SlaveDMAChangeDataSize(AudioResolution);
      SAI_SlaveEnable();
    }
  }
  
  else
  {
    SAI_Init_TDM(SAI_MASTER, AudioResolution, 2 * ChannelsPairs, BclkFsRatioMode);
    SAI_MasterDMAChangeDataSize(AudioResolution);
  }
  
  SAI_MasterEnable();
}
//------------------------------------------------------------------------------
void AudioOutMute(uint8_t MuteFlag)
{
  SAI_MasterMute(MuteFlag);
  
  if (SAICountUsed == 2)
    SAI_SlaveMute(MuteFlag);
}
//------------------------------------------------------------------------------
uint16_t GetRemainingTxSize(void)
{
  uint16_t TxSize = SAI_GetRemainingTxSize();
  
  if (AudioConfiguration >= AUDIO_CONFIG_TDM_START_INDEX)
    TxSize /= ChannelsPairs;
  
  return TxSize;
}
//------------------------------------------------------------------------------
uint16_t GetLastTxSize(void)
{
  uint16_t TxSize = SAI_GetLastTransferSize();
  
  if (AudioConfiguration >= AUDIO_CONFIG_TDM_START_INDEX)
    TxSize /= ChannelsPairs;
  
  return TxSize;
}
//------------------------------------------------------------------------------
void AudioOutInit(uint32_t AudioFrequency, uint8_t AudioResolution)
{
  if (SyncMode == MASTER_INT_SYNC)
    SAI_PLLSAIConfig(AudioFrequency);
  else if (SyncMode == MASTER_EXT_SYNC)
    SAI_ExternalSyncInit();
  
  SAI_MasterGPIOInit();
  SAI_MasterDMAInit(AudioResolution);
  
  if (AudioConfiguration < AUDIO_CONFIG_TDM_START_INDEX)
  {
    SAI_MasterInit_I2S(AudioResolution, BclkFsRatioMode);
  
    if (SAICountUsed == 2)
    {
      SAI_SlaveGPIOInit();
      SAI_SlaveDMAInit(AudioResolution);
      SAI_SlaveInit(AudioResolution, BclkFsRatioMode);
      SAI_SlaveEnable();
    }
  }
  else
    SAI_MasterInit_TDM(AudioResolution, 2 * ChannelsPairs, BclkFsRatioMode);
    
  if (SyncMode == MASTER_INT_SYNC)
    SAI_ChangeFrequency(AudioFrequency);
  else if (SyncMode == SLAVE_SYNC)
    SAI_MasterSetSlaveSync();
  
  ExtSyncSelectSource(AudioFrequency);
  SAI_MasterEnable();
}
//------------------------------------------------------------------------------
void PlayDescriptionInit(AUDIO_Description_t *Description)
{
  uint8_t AudioConf = AudioConfiguration;
  
  if (AudioConf >= AUDIO_CONFIG_TDM_START_INDEX)
    AudioConf -= AUDIO_CONFIG_TDM_START_INDEX;
  
  switch(AudioConf)
  {
    case AUDIO_CONFIG_2_0_STEREO_32_BIT:
    case AUDIO_CONFIG_2_0_STEREO:
    default:
      Description->channels_count = CONFIG_2_0_STEREO_CHANNEL_COUNT;
      Description->channels_map = CONFIG_2_0_STEREO_CHANNEL_MAP;
    break;
    
    case AUDIO_CONFIG_3_1:
      Description->channels_count = CONFIG_3_1_CHANNEL_COUNT;
      Description->channels_map = CONFIG_3_1_CHANNEL_MAP;
    break;
    
    case AUDIO_CONFIG_4_0_QUADRO:
      Description->channels_count = CONFIG_4_0_QUADRO_CHANNEL_COUNT;
      Description->channels_map = CONFIG_4_0_QUADRO_CHANNEL_MAP;
    break;
    
    case AUDIO_CONFIG_7_1:
      Description->channels_count = CONFIG_7_1_SURROUND_CHANNEL_COUNT;
      Description->channels_map = CONFIG_7_1_SURROUND_CHANNEL_MAP;
    break;
  }
    
  Description->resolution = CONFIG_RES_BYTE_24;//CONFIG_RES_BYTE_16;
  Description->frequency = CONFIG_2_0_FREQUENCY_DEFAULT;
  Description->audio_type = USBD_AUDIO_FORMAT_TYPE_PCM;
  Description->audio_volume_db_256 = VOLUME_SPEAKER_DEFAULT_DB_256;
  Description->audio_mute = 0;
  
  if (AudioConf == AUDIO_CONFIG_2_0_STEREO_32_BIT)
  {
    Description->resolution = CONFIG_RES_BYTE_32;
    Description->frequency = CONFIG_2_0_32BIT_FREQUENCY_DEFAULT;
  }
  else if ((AudioConfiguration == AUDIO_CONFIG_7_1) || (AudioConfiguration == AUDIO_CONFIG_7_1_TDM))
    Description->resolution = CONFIG_RES_BYTE_16;
}
//------------------------------------------------------------------------------
uint8_t GetAudioConfiguration(void)
{
  return AudioConfiguration;
}
//------------------------------------------------------------------------------
void MakeSerialNumber(uint32_t *Buffer)
{
  Buffer[0] = (*(uint32_t*)DEVICE_ID1) + (*(uint32_t*)DEVICE_ID3);
  Buffer[1] = (((*(uint32_t*)DEVICE_ID2) >> 16) + AudioConfiguration) << 16;
}
//------------------------------------------------------------------------------
void AudioConfig_Init(void)
{
  AudioConfiguration |= (CONFIG_1_GPIO->IDR & (1 << CONFIG_1_PIN)) >> CONFIG_1_PIN;
  AudioConfiguration |= ((CONFIG_2_GPIO->IDR & (1 << CONFIG_2_PIN)) >> CONFIG_2_PIN) << 1;
  AudioConfiguration |= ((CONFIG_4_GPIO->IDR & (1 << CONFIG_4_PIN)) >> CONFIG_4_PIN) << 2;
  
  switch(AudioConfiguration)
  {
    case (CONFIG_PIN_2_MASK | CONFIG_PIN_3_MASK):
      AudioConfiguration = AUDIO_CONFIG_2_0_STEREO_32_BIT;
      SAICountUsed = 1;
      ChannelsPairs = 1;
    break;
   
    case CONFIG_PINS_MASK:
    default:
      AudioConfiguration = AUDIO_CONFIG_2_0_STEREO;
      SAICountUsed = 1;
      ChannelsPairs = 1;
    break;
    
    case (CONFIG_PIN_1_MASK | CONFIG_PIN_3_MASK):
      AudioConfiguration = AUDIO_CONFIG_3_1;
      ChannelsPairs = 2;
    break;
    
    case CONFIG_PIN_3_MASK:
      AudioConfiguration = AUDIO_CONFIG_4_0_QUADRO;
      ChannelsPairs = 2;
    break;
    
    case CONFIG_PIN_2_MASK:
      AudioConfiguration = AUDIO_CONFIG_7_1;
      ChannelsPairs = 4;
    break;
  }
  
  if ((TDM_MODE_GPIO->IDR & (1 << TDM_MODE_PIN)) != (1 << TDM_MODE_PIN))
  {
    AudioConfiguration += AUDIO_CONFIG_TDM_START_INDEX;
    SAICountUsed = 1;
  }
  
  uint8_t SAISyncMode = SYNC_MODE_MASK;
  if ((CONFIG_3_GPIO->IDR & (1 << CONFIG_3_PIN)) != (1 << CONFIG_3_PIN))
  {
    SAISyncMode &= ~SYNC_MODE_1_MASK;
  }
  if ((SYNC_MODE_GPIO_2->IDR & (1 << SYNC_MODE_PIN_2)) != (1 << SYNC_MODE_PIN_2))
    SAISyncMode &= ~SYNC_MODE_2_MASK;
  
  switch (SAISyncMode)
  {
    case SYNC_MODE_MASK:
    default:
      SyncMode = MASTER_INT_SYNC;
    break;
    
    case SYNC_MODE_2_MASK:
      SyncMode = MASTER_EXT_SYNC;
    break;
    
    case SYNC_MODE_1_MASK:
      SyncMode = SLAVE_SYNC;
    break;
  }
  
  if ((BCLK_Fs_RATIO_GPIO->IDR & (1 << BCLK_Fs_RATIO_PIN)) != (1 << BCLK_Fs_RATIO_PIN))
    BclkFsRatioMode = BCLK_Fs_FIXED;
}
//------------------------------------------------------------------------------
void OUTClk_Init(void)
{
  uint32_t AudioFreq = CONFIG_2_0_FREQUENCY_DEFAULT;
  uint8_t AudioRes = CONFIG_RES_BIT_24;
  
  if (SyncMode != MASTER_EXT_SYNC)
  {
    switch((OUT_CLK_CONFIG_1_GPIO->IDR & ((1 << OUT_CLK_CONFIG_1_PIN) | (1 << OUT_CLK_CONFIG_3_PIN))))
    {
      case OUT_CLK_CONFIG_1:
        AudioFreq = USB_AUDIO_CONFIG_FREQ_44_1_K;
      break;
      
      case OUT_CLK_CONFIG_2:
      default:
        AudioFreq = USB_AUDIO_CONFIG_FREQ_48_K;
      break;
      
      case OUT_CLK_CONFIG_3:
        AudioFreq = USB_AUDIO_CONFIG_FREQ_96_K;
      break;
      
      case OUT_CLK_CONFIG_4:
        AudioFreq = USB_AUDIO_CONFIG_FREQ_192_K;
      break;
    }
  }
  
  if ((AudioConfiguration == AUDIO_CONFIG_7_1) || (AudioConfiguration == AUDIO_CONFIG_7_1_TDM))
    AudioRes = CONFIG_RES_BIT_16;    
    
  AudioOutInit(AudioFreq, AudioRes);
}
//------------------------------------------------------------------------------
void ExtPowerDisable(void)
{
  POWER_DISABLE_GPIO->ODR |= 1 << POWER_DISABLE_PIN;
}
//------------------------------------------------------------------------------
void ConfigGPIOs_Init(void)
{
  CONFIG_1_GPIO->PUPDR |= 1 << (2 * CONFIG_1_PIN);
  CONFIG_2_GPIO->PUPDR |= (1 << (2 * CONFIG_2_PIN)) | (1 << (2 * CONFIG_3_PIN)) | (1 << (2 * TDM_MODE_PIN));
  while(((CONFIG_1_GPIO->PUPDR & (1 << (2 * CONFIG_1_PIN))) != (1 << (2 * CONFIG_1_PIN)))
     || ((CONFIG_2_GPIO->PUPDR & ((1 << (2 * CONFIG_2_PIN)) | (1 << (2 * CONFIG_3_PIN)) | (1 << (2 * TDM_MODE_PIN))))
         != ((1 << (2 * CONFIG_2_PIN)) | (1 << (2 * CONFIG_3_PIN)) | (1 << (2 * TDM_MODE_PIN)))));
  
  SYNC_MODE_GPIO_2->PUPDR |= 1 << (2 * SYNC_MODE_PIN_2);
  OUT_CLK_CONFIG_1_GPIO->PUPDR |= (1 << (2 * OUT_CLK_CONFIG_1_PIN)) | (1 << (2 * OUT_CLK_CONFIG_3_PIN));
    
  while((OUT_CLK_CONFIG_1_GPIO->PUPDR & ((1 << (2 * OUT_CLK_CONFIG_1_PIN)) | (1 << (2 * OUT_CLK_CONFIG_3_PIN))))
                                     != ((1 << (2 * OUT_CLK_CONFIG_1_PIN)) | (1 << (2 * OUT_CLK_CONFIG_3_PIN))));
  
  OUT_CLK_CONFIG_2_GPIO->MODER &= ~(3 << (2 * OUT_CLK_CONFIG_2_PIN));
  OUT_CLK_CONFIG_2_GPIO->MODER |= (1 << (2 * OUT_CLK_CONFIG_2_PIN)) | (1 << (2 * CONFIG_5_PIN));
  while((OUT_CLK_CONFIG_2_GPIO->MODER & ((1 << (2 * OUT_CLK_CONFIG_2_PIN)) | (1 << (2 * CONFIG_5_PIN))))
                                     != ((1 << (2 * OUT_CLK_CONFIG_2_PIN)) | (1 << (2 * CONFIG_5_PIN))));
  
  POWER_DISABLE_GPIO->MODER |= 1 << (2 * POWER_DISABLE_PIN);
  
  EXT_SYNC_SELECT_GPIO->MODER &= ~(3 << (2 * EXT_SYNC_SELECT_3_PIN));
  EXT_SYNC_SELECT_GPIO->PUPDR &= ~(3 << (2 * EXT_SYNC_SELECT_3_PIN));
  EXT_SYNC_SELECT_GPIO->MODER |= (1 << (2 * EXT_SYNC_SELECT_1_PIN)) | (1 << (2 * EXT_SYNC_SELECT_2_PIN))  | (1 << (2 * EXT_SYNC_SELECT_3_PIN));
  
  BCLK_Fs_RATIO_GPIO->PUPDR |= 1 << (2 * BCLK_Fs_RATIO_PIN);
  
  CONFIG_4_GPIO->PUPDR |= 1 << (2 * CONFIG_4_PIN);
  while((CONFIG_4_GPIO->PUPDR & (1 << (2 * CONFIG_4_PIN))) != (1 << (2 * CONFIG_4_PIN)));
}

